import { html } from "uhtml";
import { signal } from "uhtml/preactive";
import { registerSW } from "virtual:pwa-register";

const open = signal(false);
const updateSW = signal<
  ((reloadPage?: boolean | undefined) => Promise<void>) | undefined
>(undefined);

export const UpdateModal = () => {
  return html` <dialog ?open=${open.value}>
    <article>
      <header>An update is available!</header>
      <p>
        A new version of the app is available. Click the button below to update
        and reload.
      </p>
      <footer>
        <button aria-label="Update and reload" @click=${update}>Update</button>
      </footer>
    </article>
  </dialog>`;
};

const update = (event: Event) => {
  event.preventDefault();
  if (updateSW.value) {
    console.log("Service Worker is updating.");
    updateSW.value(true);
  } else {
    console.error("Service Worker failed to update.");
  }
};

export const checkForUpdates = () => {
  if ("serviceWorker" in navigator) {
    const promise = registerSW({
      onNeedRefresh() {
        console.log("Service Worker is ready to update.");
        open.value = true;
      },
      onRegisteredSW() {
        console.log("Service Worker has been registered.");
      },
      onOfflineReady() {
        console.log("Service Worker is ready to handle offline usage.");
      },
    });

    updateSW.value = promise;
  }
};
